<?php

namespace Drupal\recurly_aegir\Processors;

use Symfony\Component\HttpFoundation\Request;
use Drupal\Component\Utility\Html;
use Drupal\Core\Url;
use Drupal\Core\Link;
use Drupal\node\Entity\Node;
use Drupal\recurly_aegir\Exceptions\RecurlyAegirTaskCreationFailedException;
use Drupal\recurly_aegir\Provisioners\RecurlyAegirSiteProvisioner;
use Drupal\recurly_aegir\Provisioners\RecurlyAegirSiteCreateProvisioner;

/**
 * Processes Recurly notifications for Aegir.
 *
 * @todo Make this class and all subclasses services?
 */
abstract class RecurlyAegirProcessor {

  const NOTIFICATION_MAP = [
    'new_account_notification' => 'RecurlyAegirAccountNewProcessor',
    'canceled_account_notification' => 'RecurlyAegirAccountCanceledProcessor',
    'billing_info_updated_notification' => 'RecurlyAegirAccountUpdatedProcessor',
    'reactivated_account_notification' => 'RecurlyAegirAccountReactivatedProcessor',
    'new_invoice_notification' => 'RecurlyAegirInvoiceNewProcessor',
    'processing_invoice_notification' => 'RecurlyAegirInvoiceProcessingProcessor',
    'closed_invoice_notification' => 'RecurlyAegirInvoiceClosedProcessor',
    'past_due_invoice_notification' => 'RecurlyAegirInvoiceOverdueProcessor',
    'new_subscription_notification' => 'RecurlyAegirSubscriptionNewProcessor',
    'updated_subscription_notification' => 'RecurlyAegirSubscriptionUpdatedProcessor',
    'canceled_subscription_notification' => 'RecurlyAegirSubscriptionCanceledProcessor',
    'expired_subscription_notification' => 'RecurlyAegirSubscriptionExpiredProcessor',
    'renewed_subscription_notification' => 'RecurlyAegirSubscriptionRenewedProcessor',
    'scheduled_payment_notification' => 'RecurlyAegirPaymentScheduledProcessor',
    'processing_payment_notification' => 'RecurlyAegirPaymentProcessingProcessor',
    'successful_payment_notification' => 'RecurlyAegirPaymentSuccessfulProcessor',
    'failed_payment_notification' => 'RecurlyAegirPaymentFailedProcessor',
    'successful_refund_notification' => 'RecurlyAegirPaymentRefundSuccessfulProcessor',
    'void_payment_notification' => 'RecurlyAegirPaymentVoidProcessor',
  ];

  /**
   * Account information.
   *
   * @var SimpleXMLElement|null
   */
  protected $account;

  /**
   * Subscription information.
   *
   * @var SimpleXMLElement|null
   */
  protected $subscription;

  /**
   * Transaction information.
   *
   * @var SimpleXMLElement|null
   */
  protected $transaction;

  /**
   * Invoice information.
   *
   * @var SimpleXMLElement|null
   */
  protected $invoice;

  protected $result = FALSE;

  /**
   * Class constuctor.
   *
   * @param SimpleXMLElement|null $account
   *   Account information.
   * @param SimpleXMLElement|null $subscription
   *   Subscription information.
   * @param SimpleXMLElement|null $transaction
   *   Transaction information.
   * @param SimpleXMLElement|null $invoice
   *   Invoice information.
   */
  public function __construct(
      $account = NULL,
      $subscription = NULL,
      $transaction = NULL,
      $invoice = NULL) {
    $this->account = $account;
    $this->subscription = $subscription;
    $this->transaction = $transaction;
    $this->invoice = $invoice;
  }

  /**
   * Process any notifications coming from Recurly.
   *
   * Must be overriden by subclasses to actually do something.
   *
   * @return $this
   *   The object itself.
   */
  abstract public function process();

  /**
   * Get the processing result.
   *
   * @return bool
   *   TRUE if processing was successful; FALSE otherwise.
   */
  public function getResult() {
    return $this->result;
  }

  /**
   * Gets the Recurly subscription URL from a UUID.
   *
   * @param string $uuid
   *   The subscription identifier.
   *
   * @return string
   *   The subscription URL on the Recurly subdomain.
   */
  public static function getSubscriptionUrl($uuid) {
    $subdomain = \Drupal::config('recurly.settings')->get('recurly_subdomain');
    return 'https://' . $subdomain . '.recurly.com/subscriptions/' . $uuid;
  }

  /**
   * Gets a site's subscription ID.
   *
   * @param Drupal\node\Entity\Node $site
   *   The site whose ID is being requested.
   *
   * @return string
   *   The subscription's UUID.
   */
  public static function getSubscriptionId(Node $site) {
    return $site->get('field_site_subscription_url')->getValue()[0]['title'];
  }

  /**
   * Gets the configuration link for a specific site.
   *
   * @param Drupal\node\Entity\Node $site
   *   The site node.
   *
   * @return Drupal\Core\Link
   *   The site node's configuration link.
   */
  public static function getSiteConfigurationLink(Node $site) {
    $url = Url::fromRoute('entity.node.edit_form', ['node' => $site->id()]);
    return Link::fromTextAndUrl(t('<em>Configuration required!</em>'), $url);
  }

  /**
   * Gets the link for a specific client site.
   *
   * @param Drupal\node\Entity\Node $site
   *   The site node.
   *
   * @return Drupal\Core\Link
   *   A link to the site itself.
   */
  public static function getSiteLink(Node $site) {
    $protocol = \Drupal::request()->getScheme();
    $subdomain = Html::escape($site->getTitle());
    $domain = \Drupal::request()->getHost();
    $hostname = "$subdomain.$domain";
    $url = Url::fromUri("$protocol://$hostname/");
    return Link::fromTextAndUrl($hostname, $url);
  }

  /**
   * Launch a provisioner for a specific site.
   *
   * @param Drupal\node\Entity\Node $site
   *   The site to be provisioned.
   * @param Drupal\recurly_aegir\Provisioners\RecurlyAegirSiteProvisioner $provisioner
   *   The provisioner doing the provisioning.
   */
  public static function launchSiteProvisioner(Node $site, RecurlyAegirSiteProvisioner $provisioner) {
    try {
      $task_id = $provisioner->provision()->getTaskId();

      // Save a link to the task ID for future reference.
      $aegir_url = Request::create($provisioner->getAegirServiceEndpoint())->getSchemeAndHttpHost();
      $site->field_site_tasks->appendItem([
        'uri' => $aegir_url . '/hosting/task/' . $task_id,
        'title' => date("Y-m-d \@ H:i:s") . ' » ' . $provisioner->getTaskType(),
      ]);

      // Log the successful provisioning task creation along with its ID.
      $provisioner_name = (new \ReflectionClass($provisioner))->getShortName();
      \Drupal::logger('recurly_aegir')
        ->info('Remote task %task created to provision %hostname for %client from %template via %provisioner.', [
          '%task' => $task_id,
          '%hostname' => $provisioner->getHostname(),
          '%client' => method_exists($provisioner, 'getClient') ? $provisioner->getClient()->getDisplayName() : 'NO CLIENT',
          '%template' => method_exists($provisioner, 'getTemplate') ? $provisioner->getTemplate() : 'NO TEMPLATE',
          '%provisioner' => $provisioner_name,
        ]);

      // Update the user as to the status of his/her new site (if applicable).
      if (in_array($provisioner_name, RecurlyAegirSiteCreateProvisioner::PROVISIONERS_CREATE)) {
        drupal_set_message(t('Your new site %site is in the process of being provisioned. This usually takes several minutes. When complete, you will receive an e-mail with instructions for logging into it.', [
          '%site' => $provisioner->getHostname(),
        ]));
      }
    }
    catch (RecurlyAegirTaskCreationFailedException $e) {
      watchdog_exception('recurly_aegir', $e);
    }
  }

  /**
   * Determines if a particular subscription is active for a site.
   *
   * @param Drupal\node\Entity\Node $site
   *   The site in question.
   *
   * @return bool
   *   TRUE if subscription is active; FALSE otherwise.
   */
  public static function subscriptionIsActive(Node $site) {
    if (!recurly_client_initialize()) {
      return FALSE;
    }

    try {
      $state = \Recurly_Subscription::get(self::getSubscriptionId($site))->state;
      if ($state == 'active') {
        return TRUE;
      }
      return FALSE;
    }
    catch (\Exception $e) {
      return FALSE;
    }
  }

}
