<?php

namespace Drupal\recurly_aegir\Provisioners;

use Drupal\Component\Utility\UrlHelper;
use Drupal\Core\Form\FormStateInterface;
use Drupal\user\Entity\User;
use Drupal\recurly_aegir\Provisioners\RecurlyAegirSiteProvisioner;
use Drupal\recurly_aegir\Provisioners\RecurlyAegirSiteListProvisioner;

/**
 * Class for creating new sites via Aegir's Web service API.
 */
abstract class RecurlyAegirSiteCreateProvisioner extends RecurlyAegirSiteProvisioner {

  /**
   * The list of provisioner classes available for creating new sites.
   *
   * This won't be necessary when we switch to plug-in discovery.
   */
  const PROVISIONERS_CREATE = [
    'RecurlyAegirSiteCloneProvisioner',
    'RecurlyAegirSiteInstallProvisioner',
  ];

  /**
   * @var User $client
   *   Provisioning is on behalf of this client user.
   */
  protected $client;

  /**
   * @var string $template
   *   The machine name of the template to use in provisioning the site.
   */
  protected $template;

  /**
   * {@inheritdoc}
   *
   * @param User $client
   *   Client user details for creating the initial Drupal account on a new site.
   * @param string $template
   *   The machine name of the template to use in provisioning the site.
   */
  public function __construct($hostname, User $client, $template) {
    parent::__construct($hostname);

    if (empty($client)) {
      throw new \Exception('Site-creation provisioners must be provided with a User on construction.');
    }
    if (empty($template)) {
      throw new \Exception('Site-creation provisioners must be provided with a template ID on construction.');
    }

    $this->client = $client;
    $this->template = $template;
  }

  /**
   * {@inheritdoc}
   */
  public static function getTaskType() {
    return static::TASK_TYPE;
  }

  public function getClient() {
    return $this->client;
  }

  public function getTemplate() {
    return $this->template;
  }

  /**
   * Validates the site name chosen by the user.
   *
   * We need to ensure that the chosen name does not have special characters as
   * it's the subdomain of the FQDN, and forms a URL. To test, verify that the
   * name is the same if it's URL encoded. Also, ensure the name isn't taken.
   *
   * @param array $element
   *   The form element to process.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state.
   * @param array $complete_form
   *   The complete form structure.
   *
   * @see Drupal\Core\Render\Element\Url::validateUrl()
   */
  public static function validateNewSiteName(&$element, FormStateInterface $form_state, &$complete_form) {
    $site_name = trim($element['#value']);
    $form_state->setValueForElement($element, $site_name);

    if ($site_name === '' || (UrlHelper::encodePath($site_name) !== $site_name)) {
      $form_state->setError($element, t('The site name %subdomain is not valid. Special characters are not allowed.', [
        '%subdomain' => $site_name,
      ]));
    }

    if (!self::siteNameIsAvailable($site_name)) {
      $form_state->setError($element, t('The site name %subdomain is already taken. Please choose another.', [
        '%subdomain' => $site_name,
      ]));
    }
  }

  public static function siteNameIsAvailable($site_name) {
    $enabled_sites = [];
    $provisioner = new RecurlyAegirSiteListProvisioner($site_name);

    foreach ($provisioner->provision()->getSiteList() as $site) {
      if ($site['status'] == 1) {
        $enabled_sites[] = $site['title'];
      }
    }

    if (in_array($site_name . '.' . \Drupal::request()->getHost(), $enabled_sites)) {
      return FALSE;
    }
    return TRUE;
  }

}
