<?php

namespace Drupal\recurly_aegir\Provisioners;

use Drupal\recurly_aegir\Exceptions\RecurlyAegirTaskCreationFailedException;

/**
 * Base class for provisioning new sites via Aegir's Web service API.
 *
 * @todo Make this class a plugin manager and make the subclasses plugins.
 */
abstract class RecurlyAegirSiteProvisioner {

  /**
   * The virtual hostname of the site to provision.
   *
   * @var string
   */
  protected $hostname;

  /**
   * The response data received after sending a provision request.
   *
   * @var array
   */
  protected $response;

  /**
   * The response headers received after sending a provision request.
   *
   * @var array
   */
  protected $headers;

  /**
   * The response code received after sending a provision request.
   *
   * @var int
   */
  protected $status;

  /**
   * Class Constructor.
   *
   * @param string $hostname
   *   The virtual hostname of the site to provision.
   */
  public function __construct($hostname) {
    if (empty($hostname)) {
      throw new \Exception('Provisioners must be provided with a hostname on construction.');
    }

    $this->hostname = $hostname;
  }

  /**
   * Gets the name of the task type used to provision a new site.
   *
   * @return string
   *   The name of the task type.
   */
  abstract public static function getTaskType();

  /**
   * Sends a request to Aegir to provision a site.
   *
   * Implementing classes must do the following:
   *   1. Make a service call to Aegir that creates a task to provision a site.
   *   2. Set $this->$task_id to the task ID returned in the call (if possible).
   *   3. Return $this for method chaining.
   *
   * @return $this
   *   The object itself.
   */
  abstract public function provision();

  /**
   * Returns the endpoint for the Aegir Web service.
   *
   * @return string
   *   The endpoint URL.
   */
  public static function getAegirServiceEndpoint() {
    $endpoint = \Drupal::config('recurly_aegir.settings')->get('service_endpoint_url');
    if (empty($endpoint)) {
      \Drupal::logger('recurly_aegir')
        ->critical('The Aegir service endpoint has not been configured.');
    }
    return $endpoint;
  }

  /**
   * Sends a request to the Aegir host, and sets class variables with results.
   *
   * @param array $resource
   *   The endpoint resource to query.
   * @param array $data
   *   The data to send.
   */
  protected function sendRequestAndReceiveResponse($resource, $data) {
    $target = $this->getAegirServiceEndpoint() . '/' . $resource;

    \Drupal::logger('recurly_aegir')->info('Sending remote request for resource "%resource" with headers %headers and data "%data"', [
      '%resource' => $resource,
      '%headers' => $this->getHeadersToSend(),
      '%data' => serialize($data),
    ]);

    try {
      if (empty($data)) {
        $request = \Drupal::httpClient()->get($target, ['headers' => $this->getHeadersToSend()]);
      }
      else {
        $request = \Drupal::httpClient()->post($target, [
          'headers' => $this->getHeadersToSend(),
          'json' => $data,
        ]);
      }
    }
    catch (Exception $e) {
      watchdog_exception('recurly_aegir', $e->getMessage());
    }

    $this->status = $request->getStatusCode();
    $this->headers = $request->getHeaders();
    $this->response = json_decode((string) $request->getBody(), TRUE);
  }

  protected function getHeadersToSend() {
    return [
      'API-KEY' => \Drupal::config('recurly_aegir.settings')->get('service_endpoint_key'),
    ];
  }

  /**
   * Returns the Aegir task ID returned after provisioning a site.
   *
   * @return int
   *   The task ID.
   *
   * @throws RecurlyAegirTaskCreationFailedException
   *   If there is no task ID because the service call failed.
   *
   * @see $this->provision()
   */
  public function getTaskId() {
    if (empty($this->response['nid'])) {
      throw new RecurlyAegirTaskCreationFailedException(sprintf(
        'Task creation failed for %s via %s with template %s.',
        $this->hostname,
        (new \ReflectionClass(get_class($this)))->getShortName(),
        property_exists($this, 'template') ? $this->template : 'NO TEMPLATE'
      ));
    }
    return $this->response['nid'];
  }

  /**
   * Returns the target hostname associated with this provisioner.
   */
  public function getHostname() {
    return $this->hostname;
  }

}
